ADB_COMMAND_SETTING_SHOW_TOUCHES = 'settings put system show_touches '
ADB_COMMAND_SETTING_POINTER_LOCATION = 'settings put system pointer_location '

ADB_COMMAND_GET_SOC_ARCHITECTURE = 'getprop ro.product.cpu.abi'
