if __name__ != '__main__':
    from .single_functions import *
    from .int_to_chinese import int_to_chinese
    from .AyDict import AyDict
    from .AyStr import AyStr
    from .get_file_format import get_file_format
    from .HiddenPrints import HiddenPrints


if __name__ == '__main__':
    print('This is a module, not a script.')
